class URI:
    def __init__(self, host: str, unformatted_path: str, **uri_params):
        self.host = host
        self.formatted = unformatted_path.format(**uri_params)
        self.full = f"{self.host}{self.formatted}"
        self.raw = unformatted_path
        self.uri_params: dict = uri_params



from requests import Response

from swagger_coverage_py.configs import API_DOCS_TYPE, API_DOCS_VERSION
from swagger_coverage_py.results_writers.base_schemas_manager import ApiDocsManagerBase
from swagger_coverage_py.uri import URI


class OpenApiSchemasManager(ApiDocsManagerBase):
    def __init__(self, uri: URI, method: str, response: Response, kwargs: dict):
        super().__init__(uri, response, kwargs, method)

    def _paths(self):
        path_ = self._uri.raw.split("?")[0]
        dict_ = {
            path_: {
                self._method: {
                    "parameters": self._get_path_params() + self._get_query_params(),
                    "responses": {self._response.status_code: {}},
                }
            }
        }

        body_params = self._get_body_params()
        if body_params:
            dict_[path_][self._method]["requestBody"] = body_params
        return dict_

    def _get_schema(self):
        schema_dict = {
            API_DOCS_TYPE: API_DOCS_VERSION,
            "info": {"title": "Recorded Request"},
            "paths": self._paths(),
        }
        return schema_dict



import os
import platform
import re
import shutil
from pathlib import Path

import requests

from swagger_coverage_py.configs import API_DOCS_FORMAT
from swagger_coverage_py.docs_writers.api_doc_writer import write_api_doc_to_file


class CoverageReporter:
    def __init__(self, api_name: str, host: str):
        self.host = host
        self.swagger_doc_file = f"swagger-doc-{api_name}.{API_DOCS_FORMAT}"
        self.output_dir = self.__get_output_dir()
        self.ignore_requests = []
        self.swagger_coverage_config = f"swagger-coverage-config-{api_name}.json"

    def __get_output_dir(self):
        output_dir = "swagger-coverage-output"
        subdir = re.match(r"(^\w*)://(.*)", self.host).group(2)
        return f"{output_dir}/{subdir}"

    def setup(
        self, path_to_swagger_json: str, auth: object = None, cookies: dict = None
    ):
        """Setup all required attributes to generate report

        :param path_to_swagger_json: The relative URL path to the swagger.json (example: "/docs/api")
        :param auth: Authentication object acceptable by "requests" library
        :param cookies: Cookies dictionary. (Usage example: set this to bypass Okta auth locally)

        """
        link_to_swagger_json = f"{self.host}{path_to_swagger_json}"

        response = requests.get(link_to_swagger_json, auth=auth, cookies=cookies)
        assert response.ok, (
            f"Swagger doc is not pulled. See details: "
            f"{response.status_code} {response.request.url}"
            f"{response.content}\n{response.content}"
        )

        write_api_doc_to_file(self.swagger_doc_file, response)

    def generate_report(self):
        inner_location = "swagger-coverage-commandline/bin/swagger-coverage-commandline"

        cmd_path = os.path.join(os.path.dirname(__file__), inner_location)
        assert Path(
            cmd_path
        ).exists(), (
            f"No commandline tools is found in following locations:\n{cmd_path}\n"
        )

        if self.swagger_coverage_config:
            command = f"{cmd_path} -s {self.swagger_doc_file} -i {self.output_dir} -c {self.swagger_coverage_config}"
        else:
            command = f"{cmd_path} -s {self.swagger_doc_file} -i {self.output_dir}"

        command = (
            command if platform.system() != "Windows" else command.replace("/", "\\")
        )

        os.system(command)

    def cleanup_input_files(self):
        shutil.rmtree(self.output_dir, ignore_errors=True)
        Path(self.output_dir).mkdir(parents=True, exist_ok=True)



import json

import requests
import yaml

from swagger_coverage_py.configs import API_DOCS_FORMAT, API_DOCS_TYPE


def __write_api_doc_to_json(file_path: str, api_doc_data: requests.Response):
    api_doc_data = api_doc_data.json()
    if API_DOCS_TYPE == "swagger" and not api_doc_data.get("swagger", None):
        api_doc_data["swagger"] = "2.0"

    with open(file_path, "w+") as file:
        file.write(json.dumps(api_doc_data))


def __write_api_doc_to_yaml(file_path: str, api_doc_data: requests.Response):
    data = yaml.safe_load(str(api_doc_data.text))

    if API_DOCS_TYPE == "swagger" and not data.get("swagger", None):
        data["swagger"] = "2.0"

    with open(file_path, "w+") as file:
        file.write(yaml.safe_dump(data, indent=4, sort_keys=False))


def write_api_doc_to_file(file_path: str, api_doc_data: requests.Response):
    if API_DOCS_FORMAT == "json":
        __write_api_doc_to_json(file_path, api_doc_data)
    else:
        __write_api_doc_to_yaml(file_path, api_doc_data)



import re
from typing import List

from requests import Response

from swagger_coverage_py.configs import API_DOCS_TYPE, API_DOCS_VERSION
from swagger_coverage_py.results_writers.base_schemas_manager import ApiDocsManagerBase
from swagger_coverage_py.uri import URI


class SwaggerSchemasManager(ApiDocsManagerBase):
    def __init__(self, uri: URI, method: str, response: Response, kwargs: dict):
        super().__init__(uri, response, kwargs, method)
        self._uri = uri
        self.__response = response

    def __host(self):
        return self._uri.host

    def __schema(self) -> List[str]:
        return [re.match(r"(^\w*):", self._uri.host).group(1)]

    def __consumes(self) -> list:
        return [self.__response.request.headers.get("content-type", "")]

    def __produces(self) -> list:
        return [self.__response.headers.get("content-type", "")]

    def _paths(self):
        path_ = self._uri.raw.split("?")[0]
        params = (
            self._get_path_params() + self._get_query_params() + self._get_body_params()
        )
        dict_ = {
            path_: {
                self._method: {
                    "parameters": params,
                    "responses": {self.__response.status_code: {}},
                }
            }
        }
        return dict_

    def _get_schema(self):
        schema_dict = {
            API_DOCS_TYPE: API_DOCS_VERSION,
            "host": self.__host(),
            "schemes": self.__schema(),
            "consumes": self.__consumes(),
            "produces": self.__produces(),
            "paths": self._paths(),
        }
        return schema_dict



from typing import Union

from requests import Response

from swagger_coverage_py.configs import API_DOCS_TYPE
from swagger_coverage_py.results_writers.openapi_schemas_manager import (
    OpenApiSchemasManager,
)
from swagger_coverage_py.results_writers.swagger_schemas_manager import (
    SwaggerSchemasManager,
)
from swagger_coverage_py.uri import URI


class RequestSchemaHandler:
    def __init__(self, uri: URI, method: str, response: Response, kwargs: dict):
        self.__manager = self.__get_manager(uri, method, response, kwargs)

    @staticmethod
    def __get_manager(
        uri, method, response, kwargs
    ) -> Union[SwaggerSchemasManager, OpenApiSchemasManager]:
        if API_DOCS_TYPE == "swagger":
            return SwaggerSchemasManager(uri, method, response, kwargs)

        return OpenApiSchemasManager(uri, method, response, kwargs)

    def write_schema(self):
        self.__manager.write_schema()



import requests

from swagger_coverage_py.request_schema_handler import RequestSchemaHandler
from swagger_coverage_py.uri import URI


class CoverageListener:
    def __init__(
        self, method: str, base_url: str, raw_path: str, uri_params: dict, **kwargs
    ):
        """Records an HTTP request as a file in swagger format

        :param method: the HTTP method name in lowercase.
        :param base_url: Base URl with a protocol but without a path. (e.g. "https://petstore.swagger.io")
        :param raw_path: Not formatted URL path.
            Parameters names in braces will be used for further formatting  (e.g. "/v2/store/order/{orderId}")
        :param uri_params: URL path parameters. Must match to parameters names specified in "raw_path"
        :param kwargs: Optional arguments that are applicable
            for appropriate request of "requests" library. (e.g. "auth", "headers", "cookies", etc.)
        """
        self.__uri = URI(base_url, raw_path, **uri_params)
        self.response = requests.request(method, self.__uri.full, **kwargs)

        RequestSchemaHandler(self.__uri, method, self.response, kwargs).write_schema()



import os

import rootpath
from dotenv import load_dotenv

load_dotenv(f"{rootpath.detect()}/.env")

API_DOCS_TYPE = os.environ.get("API_DOCS_TYPE", "openapi")  # "swagger"
API_DOCS_VERSION = os.environ.get("API_DOCS_VERSION", "3.0.0")  # "2.0"
API_DOCS_FORMAT = os.environ.get("API_DOCS_FORMAT", "json")  # "yaml"



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="swagger-coverage",
    version="2.2.4",
    author="Jamal Zeinalov",
    author_email="jamal.zeynalov@gmail.com",
    description='Python adapter for "swagger-coverage" tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JamalZeynalov/swagger-coverage-py",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "requests>=2.25.1",
        "Faker>=6.0.0",
        "setuptools>=53.0.0",
        "PyYAML",
        "python-dotenv~=0.16.0",
        "rootpath~=0.1.1",

    ],
    python_requires=">=3.6",
    include_package_data=True,
)



import json
import os
import platform
import re

import yaml
from faker import Faker
from requests import Response

from swagger_coverage_py.configs import API_DOCS_FORMAT
from swagger_coverage_py.uri import URI


class ApiDocsManagerBase:
    def __init__(self, uri: URI, response: Response, kwargs: dict, method: str = None):
        self._uri = uri
        self._method = method
        self._response: Response = response
        self.__other_request_params = kwargs

    def _get_path_params(self) -> list:
        params_ = []
        for key, value in self._uri.uri_params.items():
            params_.append(
                {"name": key, "in": "path", "required": False, "x-example": str(value)}
            )
        return params_

    def _get_body_params(self):
        if self._response.request.body is not None:
            request_body: dict = {
                "content": {
                    "application/json": {
                        "example": json.loads(self._response.request.body)
                    }
                }
            }
        else:
            request_body = None

        return request_body

    def _get_query_params(self) -> list:
        q_params = list(self.__other_request_params.get("params", {}).items())
        raw = self._uri.raw.split("?")
        if len(raw) > 1:
            q_params += [tuple(x.split("=")) for x in str(raw[1]).split("&")]
        if not q_params:
            return []

        params_ = []
        for key, value in q_params:
            params_.append(
                {"name": key, "in": "query", "required": False, "x-example": str(value)}
            )
        return params_

    def __get_output_subdir(self):
        return re.match(r"(^\w*)://(.*)", self._uri.host).group(2)

    def write_schema(self):
        schema_dict = self._get_schema()
        rnd = Faker().pystr(min_chars=5, max_chars=5)
        file_name = f"{self._method.upper()} {self._uri.formatted[1::]}".replace(
            "/", "-"
        ).replace(":", "_")
        path_ = f"swagger-coverage-output/{self.__get_output_subdir()}"
        file_path = f"{path_}/{file_name}".split("?")[0]
        file_path = f"{file_path} ({rnd}).{API_DOCS_FORMAT}"

        try:
            with open(file_path, "w+") as file:
                if API_DOCS_FORMAT == "yaml":
                    file.write(yaml.safe_dump(schema_dict, indent=4, sort_keys=False))
                elif API_DOCS_FORMAT == "json":
                    file.write(json.dumps(schema_dict, indent=4))
                else:
                    raise Exception(
                        f"Unexpected docs format: {API_DOCS_FORMAT}. Valid formats: json, yaml"
                    )

        except FileNotFoundError as e:
            system_ = platform.system()
            abs_path = os.path.abspath(file_path)

            if system_ == "Windows" and len(abs_path) > 256:
                raise EnvironmentError(
                    f"Absolute path length is greater than 256 symbols:\n"
                    f"{abs_path}\n"
                    f"To remove this restriction you can use this guide: "
                    f"https://docs.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation#enable-long-paths-in-windows-10-version-1607-and-later "
                )
            else:
                raise Exception(
                    f"Cannot write to file.\n"
                    f"Path: {abs_path}\n"
                    f"Details: {e.strerror}"
                )

        return schema_dict
